#!/usr/bin/env python

import os
import re
import sys
import argparse

# import xlrd
import openpyxl

#import csv

#import xlsutil

#filter_headers = [
#    u"Heading 1", u"Heading 2"
#    # ... and so on
#]

filter_headers = None

def load_xslx_sheet(excel_input_filename,sheetname):
    print "Loading: " + excel_input_filename
    print "  ..."
    workbook = openpyxl.load_workbook(excel_input_filename, data_only=True)
    print "  ......Done"

    print "  Loaded worksheets: ",  workbook.sheetnames

    worksheet = None
    
    if (sheetname == None):
        print "  Selecting default/active worksheet"
        worksheet = workbook.active
    else:
        print "  Selecting worksheet '"+sheetname+"'"
        worksheet = workbook[sheetname]

    print ""
    return worksheet


def header_to_column_mapping(worksheet,header_row_pos):

    # For header line, build up hashmap of header-names to column number

    header_names_mapping = {}

    min_col = worksheet.min_column
    max_col = worksheet.max_column
    
    found_header = False
    for col_j in range(min_col,max_col+1):
        header_cell = worksheet.cell(row=header_row_pos, column=col_j)
        header_cell_value = header_cell.value.strip()
        
        if header_cell_value:
            header_names_mapping[header_cell_value] = col_j

    return header_names_mapping


def convert_worksheet_to_hashmaps(worksheet):

    print "Converting worksheet to array header-name keyed array of hashmaps"
    
    data_hashmap_array = [];

    # scan through rows until non-empty/headernames encountered

    header_row_pos = None;
    
    row_i = 1
    for row in worksheet.values:
        for value in row:
            if (value != None) :
                header_row_pos = row_i
                break
        
            print(value)
        if (header_row_pos != None): break
        row_i = row_i + 1
     
    print "  Found headername data in row: ", header_row_pos
        
    header_names_mapping = header_to_column_mapping(worksheet,header_row_pos)
    header_names = header_names_mapping.keys()

    # Move on to next row, to start processing the data values
    row_i = row_i + 1
    num_rows = worksheet.max_row;
    
    while row_i<=num_rows:
        # Work through the provided header names, building up hashmap of header-name to value
        data_hashmap = {}
        
        for header_name in header_names:

            header_col_j = header_names_mapping[header_name]

            data_cell = worksheet.cell(row=row_i,column=header_col_j)
            data_value = data_cell.value

            data_hashmap[header_name] = data_value

        row_i = row_i + 1
        
        data_hashmap_array.append(data_hashmap)

    print ""
    return data_hashmap_array

def fileset_voting_for_esc_country_in_year(data_hashmap):

    # Only generating a single fileset record here, however
    # the Greenstone format allows for this to be an array
    # of fileset entries => return [ fileset ]
    
    fileset = {}
    
    return [ fileset ]


def filter_finalist_votes(entry):
    return entry.get('(semi-) final') == "f" and entry.get('Duplicate') == None

def sortkey_year_to_country(entry):
    return str(entry.get('Year'))+entry.get('To country')


def create_to_country_voting_groups(data_hashmap_array):

    # Example values for header-names
    #   (semi-) final: f, sf
    #   Jury or Televoting: J, T
    #   Year: 1975
    #   To country: Belgium
    #   From country: Belgium
    #   Edition: 1975f, 1975sf

    
    # Filter down to just the voting results concerning finals
    data_hashmap_array_finals = list(filter(filter_finalist_votes, data_hashmap_array))

    # Sort so array entries are grouped by the country receiving the votes in a given year
    data_hashmap_array_finals.sort(key=sortkey_year_to_country)


    # Debug output
    #
    # for data_hashmap in data_hashmap_array_finals:
    #    print data_hashmap.get('To country'), data_hashmap.get('Year'), data_hashmap.get('Points'), "(Points from " + data_hashmap.get('From country') + ")"


    # Build array of country groups
    #   A country grouping includes all the votes that country receive that year
    country_groups = []

    prev_data_hashmap = data_hashmap_array_finals[0]
    country_group = [ ]

    i = 1
    num_finals = len(data_hashmap_array_finals)

    while (i < num_finals):        
        country_group.append(prev_data_hashmap)
        data_hashmap = data_hashmap_array_finals[i]

        if (data_hashmap.get('To country') != prev_data_hashmap.get('To country')):
            # moving on to a new country group
            country_groups.append(country_group)
            country_group = [ ]
            
        prev_data_hashmap = data_hashmap
        i = i + 1
        
    country_group.append(prev_data_hashmap)        
    country_groups.append(country_group)

    return country_groups


def fileset_voting_for_esc_country_in_year(to_country_year_votes):

    # Looking to build data-structure (for output as JSON) in the form
    # { "FileSet":
    #   [
    #     { "FileName": "France1991.id" },
    #     { "Description":
    #       {
    #         "Metadata":
    #           [
    #             { "name": "Germany-J", "content": 12 }, # J = Jury Vote
    #             { "name": "Germany-T", "content": 6 },  # T = Televote (if present)
    #             ...
    #           ]
    #       }
    #     }
    #   ]
    # }

    fileset_array = []
    
    metadata_array = []

    for to_country_year_vote in to_country_year_votes:
        to_country   = to_country_year_vote.get('To country')
        year         = to_country_year_vote.get('Year')
        from_country = to_country_year_vote.get('From country')
        vote_type    = to_country_year_vote.get('Jury or Televoting')
        points       = to_country_year_vote.get('Points')

        voting_rec = { "name": from_country+"-"+vote_type, "content": points }

        metadata_array.append(voting_rec)

    id_to_country = to_country_year_votes[0].get('To country')
    id_to_country = re.sub(r'\s+', '', id_to_country)    
    id_year = to_country_year_votes[0].get('Year');
    id = id_to_country + str(id_year);
    filename_id = id + ".id"

    fileset = {
        "FileSet" : [            
            { "FileName": filename_id, },
            { "Description" : { "Metadata" : metadata_array } }
        ]
    }
              
    return fileset

    
if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('input-file.xlsx')
    parser.add_argument('output-file.json', nargs='?')
    parser.add_argument('--sheetname')
    
    args = parser.parse_args()

    excel_input_filename = getattr(args,'input-file.xlsx');
    json_output_filename = getattr(args,'output-file.json');
    sheetname = getattr(args,'sheetname');
    
    if (json_output_filename == None):
        json_output_filename = os.path.splitext(excel_input_filename)[0]+'.json'
        
    worksheet = load_xslx_sheet(excel_input_filename,sheetname)

    data_hashmap_array = convert_worksheet_to_hashmaps(worksheet)

    print "Number of data rows: ", len(data_hashmap_array)

    to_country_year_voting_groups = create_to_country_voting_groups(data_hashmap_array)

    # Debug output
    #
    # print to_country_year_voting_groups
    
    # Next step is to express the grouped by-country voting data
    # in the Greenstone JSON metadata format:
    
    # { "DirectoryMetadata":
    #   [
    #     { "FileSet":
    #       [
    #         { "FileName": "France1991.id" },
    #         { "Description":
    #           {
    #             "Metadata":
    #              [
    #                { "name": "Germany-J", "content": "12" }, # J = Jury Vote
    #                  ...
    #              ]
    #           }
    #         }
    #       ]
    #     }
    #     ...
    #    ]
    #  }

    directory_metadata = []
        
    ### TODO

    ## fix up names to match those used in DBpedia ESC
    ## provide metadata value that lists the countries-year-type to voted

    # F.Y.R.Macedonia2012
    # Serbia&Montenegro2004.id    : 35 votes
    # Bosnia&Herzegovina2005.id   : 38 votes
    # TheNetherlands 
    

    # From ESC:
    #   Republic of Macedonia (2005)
    #   Netherlands (not The Netherlands)
    #   Federal Republic of Yugoslavia (1992)
    
    
    print "Creating Greenstone JSON voting metadata for:"
    for to_country_year_votes in to_country_year_voting_groups:
        
        fileset = fileset_voting_for_esc_country_in_year(to_country_year_votes)
        directory_metadata.append(fileset)

        filename_id = fileset.get('FileSet')[0].get('FileName')
        num_countries_voting_data = len(fileset.get('FileSet')[1].get('Description').get('Metadata'))
#        print fileset.get('FileSet')[1].get('Description')
        
        print "  " + filename_id.ljust(28) + ": " + str(num_countries_voting_data) + " votes"

    greenstone_metadata_json = { "DirectoryMetadta": directory_metadata }


